-- CLCProt 3.1.5.03 for WoW Mists of Pandaria 5.4
-- by Adeya, player of Paladin DyeimiAdeya currently on Garona_US
--
--    The CLCProt 3.x files include "YOU MUST READ ME NOW" (.rtf), which has  
-- that name because it is true. :-) It is in the same folder as clcprot.lua.
--
--   The prior source code for CLC Prot 3.0 is in the files of "CLCProt 2.0
-- for MoP 5.4" by Blue Warlord. All files were downloaded from Curse.com. The 
-- code for creating the Rotation tabs is modeled on code in the extant CLCRet. 
-- Comments by Adeya are identified by the string "--dA:". 
--
--   Functions that have been added, or significantly rewritten, are noted as 
-- such. Some revisions correct evident errors; remedy apparent oversights; 
-- test for a nil or initialize a variable(s); etc. Tracking the amount of Holy
-- Power in the UpdateStatus() method was incorrect and has been remedied.
--
--   The algorithim now uses the expiration time of spell cooldowns to select  
-- the action with the highest priority that is usable, and display it as "the 
-- primary skill" or as "the secondary skill". It is not necessary to use "time 
-- remaining" on cooldowns to achieve the goal. Using expiration times is the 
-- most accurate and efficient method. If a spell expiration time is less than 
-- or equal to the expiration time of the current global cooldown, then its ex-
-- piration time is returned as zero, since it will not be on cooldown when the 
-- current global cooldown ends. 
--
-- Further notes:
--
-- (1) The API GetSpellCooldown() returns the starting time and the duration 
-- for the cooldown, which are zero if the spell is not on cooldown. The dura-
-- tion includes the effects of haste, buffs, debuffs, and procs which reduce 
-- it, so modification of its value is neither necessary nor desirable.	
--
-- (2) Using the cooldown for SoR as the global cooldown has been replaced by 
-- Global Cooldown (spell ID 61304). The duration of the -current- GDC is ob-
-- tained with GetSpellCooldown(61304). Realize that the length of the GCD is 
-- not constant, but is a variable value that is calculated at the time the GCD 
-- is triggered. The duration is returned as zero when a GCD is not in progress.
-- 
-- (3) In the UpdateStatus() method: the GCD triggered by 8 spells is affected 
-- by the Sanctity of Battle. The duration of the current GCD cannot be used 
-- since updating the current time before selecting the second skill depends 
-- upon the GCD triggered by the first skill chosen. So, when the first is one 
-- of the 8 spells, the duration of the GCD reduced by the paladin's current Me-
-- lee Haste; for other spells, the maximum duration of the GCD is assumed. 
--
--   (3) In the GetCD() method: the condition  s1 == actionId  may be true while
-- parsing the priority sequence for the second skill (s2), and the action has 
-- has already been chosen as the primary skill (s1) during the same rotation. 
-- For as, asgc, cs, cswb, hor, horwb, how, howaw, j and jsw: at present the 
-- returned expiration time is the updated c_time at the start of choosing the 
-- second skill plus the span of time between the start of each rotation and 
-- the start of the next rotation (t_rspan). For all other actions the returned
-- expiration time is the current value of the variable: later.
--
--   (4) UnitBuff() and UnitDebuff(), respectively, return the duration and ex-
-- piration time of the buff/debuff, without the starting time. I assume that 
-- the host's calculations included any effects of Haste, other buffs, debuffs, 
-- and/or procs which affected the duration of the specific buff/debuff.
--
--   (5) Spells which do not consume HP require a percentage of the character's 
-- maximum "base" mana instead. The configured minimum remaining mana in order 
-- to cast Consecration is now a percentage of the maximum. Flash of Light re-
-- quires 37.8% of base; CS 15%; J 12%; AS & Cons, respectively, consume 7% of 
-- base; the rest consume 5.4% of base or less.
--
--   (6) Various and sundry exceptions were found in the quest to enable the 
-- player to include Light's Hammer, and/or Consecration with the Glyph of Con-
-- secration, in the priority sequence without breaking the addon. All known  
-- issues have been resolved, whether others remain to be discovered. But it is 
-- probably best to use them as raid cooldowns regardless. 
--
-- ACKNOWLEDGMENTS: Thanks to Wrathblood at ElitistJerks.com whose Prot "[5.4] 
-- - EF You" is an excellent guide for tanks, and to Theck at Maintankadin for 
-- his research contributions pertaining especially to Eternal Flame and to 
-- the priorities for the Level 90 talent spells. Thanks also to Theck, Thels,
-- Worldie and other participants in the forum at Maintankadin.com for sharing 
-- their knowledge, insights and advice. 
--
-- COURTESY NOTICE: the Lua scripting language is Copyright © 1994–2014 Lua.org, 
-- PUC-Rio. The Lua license is available at http://www.lua.org/license.html
-------------------------------------------------------------------------------=
---------------("------m-e-s-s-a-g-e---m-e-a-s-u-r-e-m-e-n-t----------------------------------------------------------------------------------")--

-- don't load if class is wrong
local _, class = UnitClass("player")
if class ~= "PALADIN" then return end						--dA: prior source, also in clcprot.lua (which is loaded first)

-- global variables -------------------dA: in clcprot.lua -------------------------------------------------------------
--
--  s_sht = false				-- true while the Tier 3 Level 45 Selfless Healer talent is currently selected (added)
--  s_eft = false				-- true while the Tier 3 Level 45 Eternal Flame talent is currently selected (added)
--  s_sst = false				-- true while the Tier 3 Level 45 Sacred Shield talent is currently selected (added)
--
--  s_hat = false				-- true while the Tier 5 Level 75 Holy Avenger talent is currently selected (added)
--  s_swt = false				-- true while the Tier 5 Level 75 Sanctified Wrath talent is currently selected (added)
--  s_dpt = false				-- true while the Tier 5 Level 75 Divine Purpose talent is currently selected (added)
--
--  s_hprt = false				-- true while the Tier 6 Level 90 Holy Prism talent is currently selected (added)
--  s_lht  = false				-- true while the Tier 6 Level 90 Light's Hammer talent is currently selected (added)
--  s_est  = false				-- true while the Tier 6 Level 90 Execution Sentence talent is currently selected (added)
--
--  s_goc = false				-- true while the Glyph of Consecration is in effect (added)
--  consId = 26573 				-- Consecration ID, depending upon whether GoC is in effect (without glyph is default)
--  s_gfw = false				-- true while the Glyph of Final Wrath is in effect (added)
--  allowWBhpmax = false		--true while applying or refreshing Weakened Blows allowed if HP >=5 (added)
--    
--  prioToggled = "no"			-- value changes to "qn" or "qa" when the priority sequence switches to normal or alternate
--
--  s1_twm = false				-- true while the AoE of the spell in s1 must be chosen with a mouse (added)
--  s2_twm = false				-- true while the AoE of the spell in s2 must be chosen with a mouse (added)
--
--  rota = 0					-- tally of calls to clcprot.ProtRotation() since CLCProt was loaded (debugging)(added)
--  t_rspan = 0.000				-- time span between the start of a rotation and the start of the next rotation (added)
--

local GetTime = GetTime
--local dbg						--dA: prior source - not used in this file 
local db = db					--dA: prior source 

-- the queue
local qn = {} 			-- normal queue	
local qalt = {} 		-- alt queue	
local q 				-- working var

-- list of spellId  -----------------------------------------------------------

local gcdId 				= 61304  -- Global Cooldown (-dA: actual GCD "spell") 
local asId 					= 31935	 -- avenger's shield
local awId					= 31884  -- avenging wrath				--dA: added
local basId					= 114637 -- bastion of glory
local consGlyphId			= 54928	 -- Glyph of Consecration 
local consIdGlyphed			= 116467 -- consecration with gylph
local consIdNoGlyph			= 26573  -- consecration without glyph 	
local csId 					= 35395	 -- crusader strike
local dpId					= 90174  -- divine purpose
local efId					= 114163 -- eternal flame				--dA: added
local esId 					= 114157 -- execution sentence
local gcId					= 85416  -- grand crusader
local haId					= 105809 -- holy avenger
local horId					= 53595  -- hammer of the righteous
local howId 				= 24275	 -- hammer of wrath
local hprId 				= 114165 -- holy prism	--dA: 114852 is wrong spell
local hwId					= 119072 -- holy wrath
local jId					= 20271	 -- judgment
local lhId 					= 114158 -- light's hammer
local sasId					= 20925  -- sacred shield				--dA: cf. 148039
local sorId					= 53600	 -- shield of the righteous
local swId					= 53376	 -- sanctified wrath		 	--dA: added
local wbId					= 115798 -- weakened blows
local wogId					= 85673  -- word of glory				--dA: cf. 136494

-- other variables local to the chunk	--dA: initialized (many exceptions are caused by nils!) -----------

local s1, s2 = 0, 0				--dA: spell IDs for the primary & secondary skills (actions), respectively

local c_time = GetTime()		--dA: time at the start of the current rotation; see GetStatus() 
local later = 0.000				--dA: expiration time for actions omitted from selection as s1 or s2 (added)
local howlate = 90.0			--dA: how many seconds to add to c_time to obtain the value of later (added)
local muchlater = 0.0			--dA: more expiration time for some actions omitted from selection (added)

local gcdMin = 1.000			--dA: minimum duration of Paladin GCD (added)
local gcdDur = 1.500			--dA: duration of current GCD returned by GetGlobalCooldown()(added)
local gcdMax = 1.500			--dA: maximum duration of Paladin GCD (added)
local gcdExp = 0.000			--dA: expiration time of the current GCD returned by GetGlobalCooldown() (added)
local gcdSoB = 1.000			--dA: duration of GCD when affected by Sanctity of Battle

local c_bas_s = 0				--dA: current count for the Bastion of Glory stack
local c_hp = 0					--dA: current number of Holy Power charges
local c_hp_max = 5				--dA: maximum number of Holy Power charges 
local c_mHaste = 0.00			--dA: the paladin's current Melee Haste percentage, initial default
local sorHPmin = 3				--dA: minimum HP charges required to use SoR

local s_aw = false				--dA: true while Avenging Wrath is in effect (added)
local s_dp = false				--dA: true while Divine Purpose is in effect (changed to flag)
local s_ef = false				--dA: true while Eternal Flame is in effect (changed to flag)
local s_gc = false				--dA: true while Grand Crusader is in effect (changed to flag)
local s_ha = false				--dA: true while Holy Avenger is in effect	(changed to flag)
local s_sas = false				--dA: true while Sacred Shield is active (added)
local s_sw = false				--dA: true while Sanctified Wrath is in effect (added)
local s_wb = false				--dA: true while Weakened Blows debuff is applied (added)

local x_aw = 0.000				--dA: time that Avenging Wrath buff will expire (added)
local x_bas = 0.000				--dA: time that Bastion of Glory stack will expire (added)
local x_dp = 0.000				--dA: time that Divine Purpose proc will expire (added)
local x_ef = 0.000				--dA: time that Eternal Flame buff will expire (added)
local x_gc = 0.000				--dA: time that Grand Crusader proc will expire (added)
local x_ha = 0.000				--dA: time that Holy Avenger buff will expire (added)
local x_sas = 0.000				--dA: time that Sacred Shield buff will expire (added)
local x_wb = 0.000				--dA: time that Weakened Blows debuff will expire (added)

local r_bas_s = 0.000			--dA: time remaining for the BoG stack (count > 0) (added)
local r_dp = 0.000				--dA: time remaining for Divine Purpose proc 
local r_ef = 0.000				--dA: time remaining for Eternal Flame (added)
local r_sas = 0.000				--dA: time remaining for Sacred Shield
local r_wb = 0.000				--dA: time remaining for Weakened Blows 

local s_efc      = false		--dA: true when efc is in the priority sequence that is in use (added)
local s_efc_qn   = false		--dA: true when action efc is in the normal priority sequence (added)
local s_efc_qa   = false		--dA: true when action efc is in the alternate priority sequence (added)

-- debugging variables			--dA: all added, also useful for examining performance and relationships among buffs & spells 
local s_s1 = false				--dA: true when the skill to be chosen is the primary skill
local count_getCD = 0           --dA: tally of calls to GetCooldown() during the current session 
local count_getGCD = 0 			--dA: tally of calls to GetGlobalCooldown() during the current session 

-----------------------------------------------------------------------------------------------------------------------
-- executed during addon load -----------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------------------------

-- hrName to pass as argument for melee range checks
local hrName	= GetSpellInfo(horId)

-- buff names
local buffWB = GetSpellInfo(wbId)	--dA: returns string "Weakened Blows"

-----------------------------------------------------------------------------------------------------------------------
--[[
--dA: NOTE: enable s_s1 in clcprot.ProtRotation()

local printflag = false			--dA: true when a message is to be sent to the General tab of the Chat frame

local function GetPrintflag(mod_rate)						--dA: debugging - set interval for print() output to ChatFrame1
    if mod_rate and type(mod_rate) == "number" and mod_rate > 0 then    
		local remain = rota % mod_rate						--dA: rota is a global variable
		if remain == 0 then 
--			print("CLCProt: SetPrintflag(",mod_rate,") rota: ",rota," remainder: ",remain)
			return true
		else 
			return false	
		end
	else
		print("CLCProt: rotation [",rota,"] SetPrintflag(mod_rate) invalid argument: ",mod_rate)
	end
end
--]]

local function GetTargetHealthPercent()
	local targetHealthMax, targetHealthPercentage = 0, 0
	targetHealthMax = UnitHealthMax("target") 				--dA: returns zero if player has no target
	if targetHealthMax > 0 then								
		targetHealthPercentage = UnitHealth("target") / targetHealthMax 
	end
	return targetHealthPercentage							--dA: zero if no target 
end

local function GetCooldown(ability,id)						--dA: fundamentally revised

	--dA: GetCooldown() returns the time that the spell cooldown expires; if zero, the spell is not on cooldown.
	--dA: Zero is also returned if the cooldown expires before or at the time that the current GCD expires.

	count_getCD = count_getCD + 1							--dA: debugging variable
	local wait, start, duration, enabled, cdx = 0, 0, 0, 0, 0

	if ability and type(ability) == "string" then					 
		if id and type(id) == "number" and id > 0 then 

			start, duration, enabled = GetSpellCooldown(id) 	--dA: returns 0 for start & duration while spell is -not- on cooldown
			if enabled and enabled == 0 then return later end	--dA: enabled = 0 while spell is active but pending CD (else it returns 1)
			
			if start and duration then 							--dA: returns nils for invalid spell ID 
				cdx = start + duration				 			--dA: cdx is the time that the spell cooldown expires
				if cdx <= gcdExp then 							--dA: gcdExp is the time that the current global cooldown expires
					return 0 
				else 
					return cdx
				end
			end
			print("CLCProt R4: [",rota,"] GetCooldown() invalid id on call:", count_getCD, " ability: ", ability, " id: ", id, " s1: ", s1, " s2: ", s2)
			return later
		end
		print("CLCProt R5: [",rota,"] GetCooldown() invalid id on call:", count_getCD, " ability: ", ability, " id: ", id, " s1: ", s1, " s2: ", s2)
		return later
	end
	print("CLCProt R6: [",rota,"] GetCooldown() invalid ability on call:", count_getCD, " ability: ", ability, " id: ", id, " s1: ", s1, " s2: ", s2)
	return later
end

-----------------------------------------------------------------------------------------------------------------------
-- actions ------------------------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------------------------

actions = {													--dA: global variable used in clcprot.lua function(s)

	as = {	
		id = asId,					
		GetCD = function()
			if ( s1 ) and s1 == asId then
				return c_time + t_rspan 
			else
				return GetCooldown("as",asId)						
			end
		end,
		UpdateStatus = function()
			c_time = c_time + gcdSoB 
			if ( s_gc ) and ( s_ha ) then
				c_hp = min(c_hp_max, c_hp + 3)				--dA: AS during GC grants 3 HP during HA
			elseif ( s_gc ) then							
				c_hp = min(c_hp_max, c_hp + 1)				--dA: AS during GC without HA 
			end
			s_gc = false									--dA: assumes that players use AS when AS is s1
		end,
		info = "Avenger's Shield",
	},
	asgc = {												
		id = asId,											
		GetCD = function()
			if ( s_gc ) then 
				if ( s1 ) and s1 == asId then
					return c_time + t_rspan 
				else
					return GetCooldown("asgc",asId)
				end
			else
				return later
			end
		end,
		UpdateStatus = function()
			c_time = c_time + gcdSoB 
			if ( s_ha ) then
				c_hp = min(c_hp_max, c_hp + 3)				--dA: AS during GC grants 3 HP during HA
			else
				c_hp = min(c_hp_max, c_hp + 1)				--dA: AS during GC without HA 
			end
			s_gc = false									--dA: assumes that players use AS when AS is s1
		end,
		info = "Avenger's Shield during Grand Crusader",
	},
	cons = {												--dA: local-cast ID 26573 
		id = consId,										--dA: targeted ID 116467 
		GetCD = function()
			if ( s_goc ) and SpellIsTargeting(consId) then 	--dA: returns nil if spell is -not- waiting for player to select target
				return later 								--dA: returns -1 if spell is waiting for player to select target 
			elseif ( s1 ) and s1 == consId then 
				return c_time + t_rspan 
			elseif ( UnitPower("player",0) / UnitPowerMax("player",0) * 100 ) >= db.consMana then
				return GetCooldown("cons",consId)
			else
				return later 
			end
		end,
		UpdateStatus = function()
			c_time = c_time + gcdSoB 
		end,
		info = "Consecration",
	},
	cs = {													
		id = csId,											
		GetCD = function()
			if ( s1 ) and s1 == csId then
				return c_time + t_rspan + db.csSpan
			else
				return GetCooldown("cs",csId)
			end
		end,
		UpdateStatus = function()
			c_time = c_time + gcdSoB 
			if ( s_ha ) then 								
				c_hp = min(c_hp_max, c_hp + 3)				--dA: CS with HA 
			else
				c_hp = min(c_hp_max, c_hp + 1)				--dA: CS without HA 	
			end
		end,
		info = "Crusader Strike",
	}, 
	cswb = {												--dA: not essential, but may be useful in some fights
		id = csId,											
		GetCD = function()
			if ( s1 ) and s1 == csId then
				return c_time + t_rspan + db.csSpan
			elseif ( not allowWBhpmax ) and c_hp >= c_hp_max then 
				return later
			elseif ( not s_wb ) or r_wb <= db.wbRefresh then
				if ( not s_ha ) and ( not s_sw ) then
					return GetCooldown("cswb",horId)
				elseif ( s_ha ) and c_hp <= 2 then 			--dA: avoid forgoing HP gained by using CS during HA
					return GetCooldown("cswb",horId)
				elseif ( s_sw ) and c_hp <= 2 then 			--dA: avoid forgoing HP gained by using J during SW
					return GetCooldown("cswb",horId)
				else
					return later
				end
			else
				return later 
			end
		end,
		UpdateStatus = function()
			c_time = c_time + gcdSoB 
			if ( s_ha ) then
				c_hp = min(c_hp_max, c_hp + 3)				--dA: CS with HA
			else
				c_hp = min(c_hp_max, c_hp + 1)				--dA: CS without HA
			end
		end,
		info = "Refresh Weakened Blows with Crusader Strike",
	}, 
	efc = {
		id = efId,											
		GetCD = function()
			if ( s1 ) and s1 == efId then	
				return c_time + t_rspan 
			elseif ( not s_ef ) and ( c_hp >= db.efcxpHP or ( s_dp )) then 
				return GetCooldown("efc.xp",efId)
			elseif ( s_dp ) then 		--dA: efc uses DP free cast regardless of HP, if its other conditions are true
				if r_ef <= db.efcdpRefresh and c_bas_s >= db.efcdpBoG then 
					return GetCooldown("efc.dp",efId)
				--dA: BoG Expiration 
				elseif c_bas_s >= db.efcdpBoGstk and r_bas_s >= db.r_bas_min and r_bas_s <= db.efcdpBoGxp then
					return GetCooldown("efc.dp",efId)
				else
					return later
				end
			elseif ( s_ha ) then 
				if c_hp >= db.efchaHP and r_ef <= db.efchaRefresh and c_bas_s >= db.efchaBoG then 
					return GetCooldown("efc.ha",efId)
				--dA: BoG Expiration 
				elseif c_hp >= db.efchaBoGhp and c_bas_s >= db.efchaBoGstk and r_bas_s >= db.r_bas_min and r_bas_s <= db.efchaBoGxp then
					return GetCooldown("efc.ha",efId)
				else
					return later
				end 
			elseif ( s_sw ) then 
				if c_hp >= db.efcswHP and r_ef <= db.efcswRefresh and c_bas_s >= db.efcswBoG then 
					return GetCooldown("efc.sw",efId)
				--dA: BoG Expiration 
				elseif c_hp >= db.efcswBoGhp and c_bas_s >= db.efcswBoGstk and r_bas_s >= db.r_bas_min and r_bas_s <= db.efcswBoGxp then
					return GetCooldown("efc.sw",efId)
				else
					return later
				end 
			elseif ( not s_dp ) and ( not s_ha ) and ( not s_sw ) then 
				if c_hp >= db.efcHP and r_ef <= db.efcRefresh and c_bas_s >= db.efcBoG then 
					return GetCooldown("efc",efId)
				--dA: BoG Expiration 
				elseif c_hp >= db.efcBoGhp and c_bas_s >= db.efcBoGstk and r_bas_s >= db.r_bas_min and r_bas_s <= db.efcBoGxp then
					return GetCooldown("efc",efId)
				else
					return later
				end 
			else
				return later
			end
		end,
		UpdateStatus = function()
			----------------								--dA: EF is off the GCD
			c_hp = max(0, c_hp - 3)							--dA: EF consumes all HP available, max of 3
		    c_bas_s = 0										--dA: assumes that players cast EF when s1 shows EF
		end,		
		info = "Continuous Eternal Flame with configured HP, refresh & BoG",
	},
	efg = {								--dA: efg uses DP free cast regardless of HP, if its other conditions are true
		id = efId,											
		GetCD = function()
			if ( s1 ) and s1 == efId then 
				return c_time + t_rspan 
			else
				local healthPercent = UnitHealth("player") / UnitHealthMax("player") * 100 
				if healthPercent <= 20 and ( c_hp >= 1 or ( s_dp )) then 
					return GetCooldown("efg",efId)
				elseif ( s_dp ) and c_bas_s >= db.efgdpBoG and healthPercent <= db.efgdpHealth then 
					return GetCooldown("efg.dp",efId)
				elseif ( s_ha ) and c_hp >= db.efghaHP and c_bas_s >= db.efghaBoG and healthPercent <= db.efghaHealth then 
					return GetCooldown("efg.ha",efId)
				elseif ( s_sw ) and c_hp >= db.efgswHP and c_bas_s >= db.efgswBoG and healthPercent <= db.efgswHealth then 
					return GetCooldown("efg.sw",efId)
				elseif ( not s_dp ) and ( not s_ha ) and ( not s_sw ) and c_hp >= db.efgHP and c_bas_s >= db.efgBoG and healthPercent <= db.efgHealth then 
					return GetCooldown("efg",efId)
				else
					return later
				end
			end
		end,
		UpdateStatus = function()
			----------------								--dA: EF is off the GCD
	  		c_hp = max(0, c_hp - 3)							--dA: EF consumes all HP available, max of 3
		    c_bas_s = 0										--dA: assumes that players cast EF when s1 shows EF
		end,		
		info = "Glorious Eternal Flame with configured HP, BoG & Health",
	},    
	es = {
		id = esId,
		GetCD = function()
			if ( s_est ) then 
				if ( s1 ) and s1 ~= esId then 
					return GetCooldown("es",esId)
				else
					return later
				end
			else
				return muchlater
			end
		end,
		UpdateStatus = function()
			c_time = c_time + gcdMax	
		end,
		info = "Execution Sentence",
	},
	hor = {													
		id = horId,											
		GetCD = function()
			if ( s1 ) and s1 == horId then						
				return c_time + t_rspan + db.horSpan
			else
				return GetCooldown("hor",horId)
			end
		end,
		UpdateStatus = function()
			c_time = c_time + gcdSoB 
			if ( s_ha ) then 
				c_hp = min(c_hp_max, c_hp + 3)				--dA: HotR with HA
			else
				c_hp = min(c_hp_max, c_hp + 1)				--dA: HotR without HA
			end
		end,
		info = "Hammer of the Righteous",
	},
	horwb = {												
		id = horId,											
		GetCD = function()
			if ( s1 ) and s1 == horId then
				return c_time + t_rspan + db.horSpan
			elseif ( not allowWBhpmax ) and c_hp >= c_hp_max then 
				return later
			elseif ( not s_wb ) or r_wb <= db.wbRefresh then
				if ( not s_ha ) and ( not s_sw ) then
					return GetCooldown("horwb",horId)
				elseif ( s_ha ) and c_hp <= 2 then 			--dA: avoid forgoing HP gained by using HoR during HA 
					return GetCooldown("horwb",horId)
				elseif ( s_sw ) and c_hp <= 2 then 			--dA: avoid forgoing HP gained by using J during SW
					return GetCooldown("horwb",horId)
				else
					return later
				end
			else
				return later
			end
		end,
		UpdateStatus = function()
			c_time = c_time + gcdSoB 
			if ( s_ha ) then 								--dA: HotR with HA
				c_hp = min(c_hp_max, c_hp + 3)						
			else
				c_hp = min(c_hp_max, c_hp + 1)				--dA: HotR without HA
			end
		end,
		info = "Refresh Weakened Blows with Hammer of the Righteous",
	},
	how = {	
		id = howId,
		GetCD = function()
			if IsUsableSpell(howId) then 
				if ( s1 ) and s1 == howId then
					return c_time + t_rspan + db.howSpan
				else
					return GetCooldown("how",howId)
				end
			else
				return later
			end
		end,
		UpdateStatus = function()
			c_time = c_time + gcdSoB 
		end,
		info = "Hammer of Wrath",
	},
	howaw = {
		id = howId,
		GetCD = function()
			if ( s_aw ) and IsUsableSpell(howId) then
				if ( s1 ) and s1 == howId then
					return c_time + t_rspan + db.howSpan
				else
					return GetCooldown("howaw",howId)
				end
			else
				return later
			end
		end,
		UpdateStatus = function()
			c_time = c_time + gcdSoB 
		end,
		info = "Hammer of Wrath during Avenging Wrath",
	},
	hpr = {
		id = hprId,
		GetCD = function()
			if ( s_hprt ) then 
				if ( s1 ) and s1 ~= hprId then 
					return GetCooldown("hpr",hprId)
				else
					return later
				end
			else
				return muchlater
			end
		end,
		UpdateStatus = function()
			c_time = c_time + gcdMax 
		end,
		info = "Holy Prism",
	},
	hw = {
		id = hwId,
		GetCD = function()
			if ( s1 ) and s1 ~= hwId then
				return GetCooldown("hw",hwId)
			else
				return later
			end
		end,
		UpdateStatus = function()
			c_time = c_time + gcdSoB 
		end,
		info = "Holy Wrath",
	},
	hwf = {													--dA: action added 
		id = hwId,
		GetCD = function()
			if ( not s_gfw ) then 							--dA: global flag, true while Glyph of Final Wrath is socketed
				return muchlater 
			elseif ( s1 ) and s1 == hwId then				--dA: HW already chosen for s1
				return later 
			else 
				local targetHealthPercent = GetTargetHealthPercent()	--dA: returns zero if there is no target
				if targetHealthPercent > 0 and targetHealthPercent <= 20.0 then 
					return GetCooldown("hwf",hwId)
				else
					return later
				end
			end
		end,
		UpdateStatus = function()
			c_time = c_time + gcdSoB 
		end,
		info = "Holy Wrath with Glyph of Final Wrath",
	},
	hwfaw = {												--dA: action added 
		id = hwId,
		GetCD = function()
			if ( not s_gfw ) or ( not s_aw ) then 			--dA: global flag, true while Glyph of Final Wrath is socketed
				return muchlater 
			elseif ( s1 ) and s1 == hwId then				--dA: HW already chosen for s1
				return later 
			else 
				targetHealthPercent = GetTargetHealthPercent()	--dA: returns zero if there is no target
				if targetHealthPercent > 0 and targetHealthPercent <= 20 then 
					return GetCooldown("hwfaw",hwId)
				else
					return later
				end
			end
		end,
		UpdateStatus = function()
			c_time = c_time + gcdSoB 
		end,
		info = "Holy Wrath with Glyph of Final Wrath during Avenging Wrath",
	},
	j = {													
		id = jId,											
		GetCD = function()
			if ( s1 ) and s1 ~= jId then 
				return GetCooldown("j",jId)
			elseif ( s_sw ) then 
				return c_time + t_rspan + db.jSpan
			else
				return later
			end
		end,
		UpdateStatus = function()
			c_time = c_time + gcdSoB 
			if ( s_ha ) then				
				c_hp = min(c_hp_max, c_hp + 3)				--dA: J grants 3 HP during HA, but forgoes 1 or 2 when HP > 2
			elseif ( s_sw ) then
				c_hp = min(c_hp_max, c_hp + 2)				--dA: J grants 2 HP during AW with SW, but forgoes 1 when HP > 3
			else
				c_hp = min(c_hp_max, c_hp + 1)				--dA: J without HA or SW 
			end
		end,
		info = "Judgment",
	},
	jsw = {												
		id = jId,		
		GetCD = function()
			if ( s_sw ) then 
				if ( s1 ) and s1 ~= jId then 
					return GetCooldown("j",jId)
				else
					return c_time + t_rspan + db.jSpan
				end
			else
				return later
			end
		end,
		UpdateStatus = function()
			c_time = c_time + gcdSoB 
			c_hp = min(c_hp_max, c_hp + 2)					--dA: J grants 2 HP during AW with SW, but forgoes 1 when HP > 3
		end,
		info = "Judgment during Avenging Wrath with Sanctified Wrath",
	},
	lh = {	
		id = lhId,
		GetCD = function()
			if ( s_lht ) then 
				if SpellIsTargeting(lhId) then				--dA: returns nil if spell is not waiting for player to select target
					return later							--dA: returns -1 if spell is waiting for player to select target 
				elseif ( s1 ) and s1 ~= lhId then
					return GetCooldown("lh",lhId)
				else
					return later
				end 
			else 
				return muchlater
			end 
		end,
		UpdateStatus = function()
			c_time = c_time + gcdMax
		end,
		info = "Light's Hammer",
	},
	sor = {													--dA: configurable
		id = sorId,
		GetCD = function()
			if ( s1 ) and s1 == sorId then 
				return c_time + t_rspan
			elseif ( s_efc ) and ( not s_ef ) and c_hp <= db.efcxpHP then
				return later
			elseif ( s_dp ) then 
				if c_hp >= db.sordpHP then	
					return GetCooldown("sor.dp",sorId)
				--dA: maintain the BoG stack - see also efc during DivPurp
				elseif c_bas_s >= db.sorBoGstk and r_bas_s >= db.r_bas_min and r_bas_s <= db.sorBoGxp then 
					return GetCooldown("sor.dp",sorId)
				elseif r_dp <= db.sor_dptr then				--dA: perchance the free cast has yet to be used 
					return GetCooldown("sor.dp",sorId)
				else
					return later 
				end 
			elseif ( s_ha ) then 
				if c_hp >= db.sorhaHP then				--dA: settings range 3 - 5 HP
					return GetCooldown("sor.ha",sorId)
				--dA: maintain the BoG stack - see also efc during HA
				elseif c_hp >= sorHPmin and c_bas_s >= db.sorBoGstk and r_bas_s >= db.r_bas_min and r_bas_s <= db.sorBoGxp then
					return GetCooldown("sor",sorId)
				else
					return later
				end
			elseif ( s_sw ) then 
				if c_hp >= db.sorswHP then		--dA: settings range 3 - 5 HP
					return GetCooldown("sor.sw",sorId)
				--dA: maintain the BoG stack - see also efc during SW
				elseif c_hp >= sorHPmin and c_bas_s >= db.sorBoGstk and r_bas_s >= db.r_bas_min and r_bas_s <= db.sorBoGxp then
					return GetCooldown("sor",sorId)
				else
					return later
				end
			elseif c_hp >= db.sorHP then 					--dA: the Ordinary case (range 3 - 5) :-)
				return GetCooldown("sor",sorId)
			--dA: maintain the BoG stack -- see also efc ordinary case
			elseif c_hp >= sorHPmin and c_bas_s >= db.sorBoGstk and r_bas_s >= db.r_bas_min and r_bas_s <= db.sorBoGxp then
				return GetCooldown("sor",sorId)
			else
				return later
			end
		end, 
		UpdateStatus = function()
			----------------								--dA: SoR is off the GCD
		    c_bas_s = c_bas_s + 1							
			if ( s_dp ) then
				s_dp = false 								--dA: assumes that player uses SoR when SoR is s1
			else 
				c_hp = max(0, c_hp - 3) 					--dA: the Ordinary case ... currently sor requires 3 HP to cast ...
			end
		end, 
		info = "Shield of the Righteous (configured)",  
	},
	sordp = {	
		id = sorId,
		GetCD = function()
			if ( s_dp ) then 
				if ( s1 ) and s1 == sorId then 
					return c_time + t_rspan
				elseif ( s_efc ) and ( not s_ef ) and c_hp <= db.efcxpHP then
					return GetCooldown("sor.dp",sorId)					
				elseif c_hp >= db.sordpHP then	
					return GetCooldown("sor.dp",sorId)
				--dA: increase or maintain the BoG stack - see also efc during DivPurp
				elseif c_bas_s >= db.sorBoGstk and r_bas_s >= db.r_bas_min and r_bas_s <= db.sorBoGxp then
					return GetCooldown("sor.dp",sorId)
				elseif r_dp <= db.sor_dptr then				--dA: perchance the free cast has yet to be used 
					return GetCooldown("sor.dp",sorId)
				else
					return later 
				end 
			else
				return muchlater
			end
		end,
		UpdateStatus = function()
			----------------								--dA: SoR is off the GCD
		    c_bas_s = c_bas_s + 1	
			s_dp = false 									--dA: assumes that player uses SoR when chosen for s1
		end,
		info = "Priortize Shield of the Righteous during Divine Purpose",
	},
	sorha = {												--dA: sorha recommended to use sor with HA 										
		id = sorId,
		GetCD = function()
			if ( s_ha ) then 
				if ( s1 ) and s1 == sorId then
					return c_time + t_rspan
				elseif ( s_efc ) and ( not s_ef ) and db.efcRefresh > 0 and c_hp <= db.efcHP then
					return later
				elseif c_hp >= db.sorhaHP then 
					return GetCooldown("sorha",sorId)
				--dA: increase or maintain the BoG stack - see also efc during HA
				elseif c_hp >= sorHPmin and c_bas_s >= db.sorBoGstk and r_bas_s >= db.r_bas_min and r_bas_s <= db.sorBoGxp then
					return GetCooldown("sor",sorId)
				else
					return later
				end
			else
				return muchlater
			end
		end,
		UpdateStatus = function()
			----------------								--dA: SoR is off the GCD
		    c_bas_s = c_bas_s + 1							
			c_hp = max(0, c_hp - 3) 						
		end,
		info = "Priortize Shield of the Righteous during Holy Avenger",
	},
	sorsw = {												
		id = sorId,
		GetCD = function() 
			if ( s_sw ) then 
				if ( s1 ) and s1 == sorId then 
					return c_time + t_rspan
				elseif ( s_efc ) and ( not s_ef ) and db.efcRefresh > 0 and c_hp <= db.efcHP then
					return later
				elseif c_hp >= db.sorswHP then
					return GetCooldown("sorsw",sorId)
				--dA: increase or maintain the BoG stack - see also efc during SW
				elseif c_hp >= sorHPmin and c_bas_s >= db.sorBoGstk and r_bas_s >= db.r_bas_min and r_bas_s <= db.sorBoGxp then
					return GetCooldown("sor",sorId)
				else
					return later
				end
			else
				return muchlater
			end
		end,
		UpdateStatus = function()
			----------------								--dA: SoR is off the GCD
			c_bas_s = c_bas_s + 1
			c_hp = max(0, c_hp - 3)
		end,
		info = "Priortize Shield of the Righteous during Sanctified Wrath",
	},
	ss = {
		id = sasId,
		GetCD = function()
			if ( s1 ) and s1 ~= sasId then 
				if ( s_sas ) and r_sas <= db.sasRefresh then
					return GetCooldown("ss",sasId)
				else
					return GetCooldown("ss",sasId)
				end 
			else
				return later
			end 
		end,
		UpdateStatus = function()
			c_time = c_time + gcdMax
			r_sas = muchlater -- fix to stop sacred shield from showing up immediately after use (--dA: prior source r_sas = 100)
		end,
		info = "Sacred Shield",
	},	
	wog = {								--dA: wog uses DP free cast regardless of HP, if its other conditions are true
		id = wogId,											
		GetCD = function()
			if ( s1 ) and s1 ~= wogId then
				local healthPercent = UnitHealth("player") / UnitHealthMax("player") * 100 
				if healthPercent <= 20 and c_hp >= 1 then 
					return GetCooldown("wog",wogId)
				elseif ( s_dp ) and c_bas_s >= db.wogdpBasStack and healthPercent <= db.wogdpPercent then
					return GetCooldown("wog.dp",wogId)
				elseif ( s_ha ) and c_hp >= db.woghaHp and c_bas_s >= db.woghaBasStack and healthPercent <= db.woghaPercent then 
					return GetCooldown("wog.ha",wogId)
				elseif ( s_sw ) and c_hp >= db.wogswHp and c_bas_s >= db.wogswBasStack and healthPercent <= db.wogswPercent then 
					return GetCooldown("wog.sw",wogId)
				elseif ( not s_dp) and ( not s_ha ) and ( not s_w ) and c_hp >= db.wogHp and c_bas_s >= db.wogBasStack and healthPercent <= db.wogPercent then
					return GetCooldown("wog",wogId)
				else
					return later
				end
			else
				return later
			end 
		end,
		UpdateStatus = function()
			----------------								--dA: WoG is off the GCD
			c_bas_s = 0
			if ( s_dp ) then 								--dA: WoG does not consume HP during DP
				s_dp = false								--dA: assumes that player uses WoG when wog is s1
			else
				c_hp = max(0, c_hp - 3)						--dA: WoG consumes all available HP, max of 3
			end 
		end,
		info = "Word of Glory with configured HP, BoG and Health",
	},
	wogdp = {
		id = wogId,
		GetCD = function()
			if ( s_dp ) and ( s1 ) and s1 ~= wogId then 
				local healthPercent = UnitHealth("player") / UnitHealthMax("player") * 100 
				if healthPercent <= 20  then 
					return GetCooldown("efg",efId)
				elseif c_bas_s >= db.wogdpBasStack and healthPercent <= db.wogdpPercent then
					return GetCooldown("wog.dp",wogId)
				else
					return later
				end
			else
				return later
			end
		end,
		UpdateStatus = function()
			----------------								--dA: WoG is off the GCD
		    c_bas_s = 0										
			----------------								--dA: WoG consumes 0 HP during DP
			s_dp = false									--dA: assumes that player uses WoG when wogdp is s1
		end,
		info = "Priortize Word of Glory during Divine Purpose",
	},
	wogha = {	
		id = wogId,
		GetCD = function()
			if ( s_ha ) and ( s1 ) and s1 ~= wogId then 
				local healthPercent = UnitHealth("player") / UnitHealthMax("player") * 100 
				if healthPercent <= 20 and c_hp >= 1 then 
					return GetCooldown("wogha",efId)
				elseif c_hp >= db.woghaHp and c_bas_s >= db.woghaBasStack and healthPercent <= db.woghaPercent then
					return GetCooldown("wogha",wogId)
				else
					return later
				end
			else
				return later 
			end 
		end,
		UpdateStatus = function()
			----------------								--dA: WoG is off the GCD
		    c_bas_s = 0										
			c_hp = max(0, c_hp - 3)							--dA: WoG consumes all available HP, max of 3
		end,
		info = "Priortize Word of Glory during Holy Avenger",
	},
	wogsw = {
		id = wogId,
		GetCD = function()
            if ( s_sw ) and ( s1 ) and s1 ~= wogId then 
				local healthPercent = UnitHealth("player") / UnitHealthMax("player") * 100 
				if healthPercent <= 20 and c_hp >= 1 then 
					return GetCooldown("wogsw",efId)
				elseif c_hp >= db.wogswHp and c_bas_s >= db.wogswBasStack and healthPercent <= db.wogswPercent then
					return GetCooldown("wogsw",wogId)
				else
					return later
				end
			else
				return later
			end 
		end,
		UpdateStatus = function()
			----------------								--dA: WoG is off the GCD
		    c_bas_s = 0										
			c_hp = max(0, c_hp - 3)							--dA: WoG consumes all available HP, max of 3
		end,
		info = "Priortize Word of Glory during Sanctified Wrath",
	},
}

clcprot.RR_actions = actions 

-----------------------------------------------------------------------------------------------------------------------
-- action priority queue & rotation functions 
-----------------------------------------------------------------------------------------------------------------------

function clcprot.RR_UpdateQueue()							--dA: function called in clcprot.lua during Init 
	
--dA: KEEP THIS FUNCTION IN THIS FILE - USING IT IN CLCPROT.LUA CAUSES !SWATTER & BLIZZ TO (FALSELY) REPORT AN "ERROR" 
----: expanded validation of the priority sequences to exclude duplicates & incompatible actions
----:

	-- set the current db
	db = clcprot.db.profile.rotation

	--dA: local to the chunk
	s_efc_qn, s_efc_qa = false, false

	local s_efg_qn, s_efg_qa = false, false
	local s_sas_qn, s_sas_qa = false, false
	local s_wog_qn, s_wog_qa = false, false
	local s_wogdp_qn, s_wogdp_qa = false, false
	local s_wogha_qn, s_wogha_qa = false, false
	local s_wogsw_qn, s_wogsw_qa = false, false
	local s_es_qn, s_es_qa = false, false
	local s_hpr_qn, s_hpr_qa = false, false
	local s_lh_qn, s_lh_qa = false, false

	if ( s_eft ) then 										--dA: the Eternal Flame talent is selected
		-- normal queue
		if db.prio == "" then db.prio = db.prio_eft end
		qn = {}
		for v in string.gmatch(db.prio, "[^ ]+") do
			if actions[v] then
				if v == "efc" and ( not s_efc_qn ) then	
					s_efc_qn = true									
					table.insert(qn, v)
				elseif v == "efg" and ( not s_efg_qn ) then 
					s_efg_qn = true 
					table.insert(qn, v) 
				elseif v == "hpr" and ( not s_hpr_qn ) then 
					s_hpr_qn = true
					table.insert(qn, v)
				elseif v == "lh" and ( not s_lh_qn ) then 
					s_lh_qn = true
					table.insert(qn, v)
				elseif v == "es" and ( not s_es_qn ) then 
					s_es_qn = true
					table.insert(qn, v)
				else
					local flag = false
					flag = string.match("efc efg ss wog wogdp wogha wogsw es hpr lh", v)
					if ( not flag ) then  
						table.insert(qn, v)  
					end
				end
			else
				print("CLCProt: invalid action in normal priority sequence excluded: ", v)
			end
		end
		db.prio = table.concat(qn, " ")

		--alternate queue
		if db.prioAlt == "" then db.prioAlt = db.prioAlt_eft end
		qalt = {}
		for v in string.gmatch(db.prioAlt, "[^ ]+") do
			if actions[v] then
				if v == "efc" and ( not s_efc_qa ) then	
					s_efc_qa = true									
					table.insert(qalt, v)
				elseif v == "efg" and ( not s_efg_qa ) then 
					s_efg_qa = true 
					table.insert(qalt, v) 
				elseif v == "hpr" and ( not s_hpr_qa ) then 
					s_hpr_qa = true
					table.insert(qalt, v)
				elseif v == "lh" and ( not s_lh_qa ) then 
					s_lh_qa = true
					table.insert(qalt, v)
				elseif v == "es" and ( not s_es_qa ) then 
					s_es_qa = true
					table.insert(qalt, v)
				else
					local flag = false
					flag = string.match("efc efg ss wog wogdp wogha wogsw es hpr lh", v)
					if ( not flag ) then  
						table.insert(qalt, v)  
					end
				end
			else
				print("CLCProt: invalid action in alternate priority sequence excluded: ", v)			
			end
		end
		db.prioAlt = table.concat(qalt, " ")

	elseif ( s_sst ) or ( s_sht ) then		--dA: Sacred Shield or Selfless Healer is currently selected

		-- normal queue
		if db.prio == "" then db.prio = db.prio_wog end
		qn = {}
		for v in string.gmatch(db.prio, "[^ ]+") do
			if actions[v] then
				if v == "ss" and ( not s_sht) and ( not s_sas_qn ) then
					s_sas_qn = true
					table.insert(qn, v) 
				elseif v == "wog" and ( not s_wog_qn ) then
					s_wog_qn = true 
					table.insert(qn, v) 
				elseif v == "wogdp" and ( not s_wogdp_qn ) then
					s_wogdp_qn = true
					table.insert(qn, v)
				elseif v == "wogha" and ( not s_wogha_qn ) then
					s_wogha_qn = true
					table.insert(qn, v)
				elseif v == "wogsw" and ( not s_wogsw_qn ) then
					s_wogsw_qn = true
					table.insert(qn, v)
				elseif v == "hpr" and ( not s_hpr_qn ) then 
					s_hpr_qn = true
					table.insert(qn, v)
				elseif v == "lh" and ( not s_lh_qn ) then 
					s_lh_qn = true
					table.insert(qn, v)
				elseif v == "es" and ( not s_es_qn ) then 
					s_es_qn = true
					table.insert(qn, v)
				else
					local flag = false
					flag = string.match("efc efg ss wog wogdp wogha wogsw es hpr lh", v)
					if ( not flag ) then  
						table.insert(qn, v)  
					end
				end
			else
				print("CLCProt: invalid action in normal priority sequence excluded: ", v)
			end
		end
		db.prio = table.concat(qn, " ")

		-- alternate queue
		if db.prioAlt == "" then db.prioAlt = db.prioAlt_wog end
		qalt = {}
		for v in string.gmatch(db.prioAlt, "[^ ]+") do
			if actions[v] then
				if v == "ss" and ( not s_sht ) and ( not s_sas_qa ) then
					s_sas_qa = true 
					table.insert(qalt, v) 
				elseif v == "wog" and ( not s_wog_qa ) then 
					s_wog_qa = true 
					table.insert(qalt, v) 
				elseif v == "wogdp" and ( not s_wogdp_qa ) then
					s_wogdp_qa = true
					table.insert(qalt, v)
				elseif v == "wogha" and ( not s_wogha_qa ) then
					s_wogha_qa = true
					table.insert(qalt, v)
				elseif v == "wogsw" and ( not s_wogsw_qa ) then
					s_wogsw_qa = true
					table.insert(qalt, v)
				elseif v == "hpr" and ( not s_hpr_qa ) then 
					s_hpr_qa = true
					table.insert(qalt, v)
				elseif v == "lh" and ( not s_lh_qa ) then 
					s_lh_qa = true
					table.insert(qalt, v)
				elseif v == "es" and ( not s_es_qa ) then 
					s_es_qa = true
					table.insert(qalt, v)
				else
					local flag = false
					flag = string.match("efc efg ss wog wogdp wogha wogsw es hpr lh", v)
					if ( not flag) then  
						table.insert(qalt, v)  
					end
				end
			else
				print("CLCProt: invalid action in alternate priority sequence excluded: ", v)
			end
		end
		db.prioAlt = table.concat(qalt, " ")

	else	---------- --dA: all action flags remain false --------------------

		-- normal queue
		qn = {}
		for v in string.gmatch(db.prio, "[^ ]+") do
			if actions[v] then
				if v == "wog" and ( not s_wog_qn ) then
					s_wog_qn = true 
					table.insert(qn, v) 
				elseif v == "wogdp" and ( not s_wogdp_qn ) then
					s_wogdp_qn = true
					table.insert(qn, v)
				elseif v == "wogha" and ( not s_wogha_qn ) then
					s_wogha_qn = true
					table.insert(qn, v)
				elseif v == "wogsw" and ( not s_wogsw_qn ) then
					s_wogsw_qn = true
					table.insert(qn, v)
				elseif v == "hpr" and ( not s_hpr_qn ) then 
					s_hpr_qn = true
					table.insert(qn, v)
				elseif v == "lh" and ( not s_lh_qn ) then 
					s_lh_qn = true
					table.insert(qn, v)
				elseif v == "es" and ( not s_es_qn ) then 
					s_es_qn = true
					table.insert(qn, v)
				else
					local flag = false
					flag = string.match("efc efg ss wog wogdp wogha wogsw es hpr lh", v)
					if ( not flag ) then  
						table.insert(qn, v)  
					end
				end
			else
				print("CLCProt: invalid action in normal priority sequence excluded: ", v)
			end
		end
		db.prio = table.concat(qn, " ")

		-- alternate queue
		qalt = {}
		for v in string.gmatch(db.prioAlt, "[^ ]+") do
			if actions[v] then
				if v == "wog" and ( not s_wog_qa ) then 
					s_wog_qa = true 
					table.insert(qalt, v) 
				elseif v == "wogdp" and ( not s_wogdp_qa ) then
					s_wogdp_qa = true
					table.insert(qalt, v)
				elseif v == "wogha" and ( not s_wogha_qa ) then
					s_wogha_qa = true
					table.insert(qalt, v)
				elseif v == "wogsw" and ( not s_wogsw_qa ) then
					s_wogsw_qa = true
					table.insert(qalt, v)
				elseif v == "hpr" and ( not s_hpr_qa ) then 
					s_hpr_qa = true
					table.insert(qalt, v)
				elseif v == "lh" and ( not s_lh_qa ) then 
					s_lh_qa = true
					table.insert(qalt, v)
				elseif v == "es" and ( not s_es_qa ) then 
					s_es_qa = true
					table.insert(qalt, v)
				else
					local flag = false
					flag = string.match("efc efg ss wog wogdp wogha wogsw es hpr lh", v)
					if ( not flag) then  
						table.insert(qalt, v)  
					end
				end
			else
				print("CLCProt: invalid action in alternate priority sequence excluded: ", v)
			end
		end
		db.prioAlt = table.concat(qalt, " ")
		if GetSpecialization() == 2 then message("CLCProt WARNING: flags were not set for talents! Check your Tier 3 Level 45 talent selection and reload the UI (/reload).") end
	end 
	return
end

local function GetDebuff(buff)

	--dA: GetBuff() returns (a) true if the buff exists, (b) time left, (c) stack count, and (d) the time that buff expires

	local d_name = "d_name"
	local d_count, d_expires = 0, 0.000
	local nflag = false
	local left = 0.000
	
	if buff and type(buff) == "string" and buff ~= "" then 
	
		d_name, _, _, d_count, _, _, d_expires = UnitDebuff("target", buff, nil, "TARGET")

		if d_name then	
			nflag = true
			left = max(0, d_expires - c_time) 
		else												--dA: debuff not found on target (d_name is nil)
			left = 0.000
			d_expires = 0.000	
		end
		return nflag, left, d_count, d_expires				--dA: time left in seconds to millisecond precision
	else
		print("CLCProt rotation [", rota, "] ERROR: buff is invalid for GetDebuff(", buff, ").")
		return nflag, left, d_count, d_expires 
	end
end

local function GetBuff(id)									--dA: prior source replaced

	--dA: GetBuff() returns (a) true if the buff exists, (b) time left, (c) stack count, and (d) the time that buff expires

	--dA: While Sanctified Wrath is active, the WoW client displays an icon for it among the buffs on the default UI. However,
	----: UnitBuff() returns zero for the Sanctified Wrath expiration time. Since it does return the name, whether Sanctified 
	----: Wrath is in effect can be flagged. 

	local left = 0.000
	local nflag = false
	if id and type(id) == "number" and id > 0 then 
		local sp_name = GetSpellInfo(id)					--dA: name is nil if the id is nonesuch
		if sp_name then

			local name, _, _, stack, _, _, expires, _, _, _, spellId = UnitBuff("player", sp_name)

			if name then 
				nflag = true								--dA: buff exists on the paladin
				if expires and expires > 0 then				--dA: time that the buff will expire, as expresed by GetTime()
					left = max(0, expires - c_time)
					if left == 0 then stack = 0 end 	
			
					if stack then							--dA: if buff does not stack, then stack = 0 (documented for UnitDebuff)
						return nflag, left, stack, expires		
					else
						return nflag, left, 0, expires
					end
				else
					return nflag, 0.000, 0, 0.000			--dA: only name and zero values are returned for Sanctified Wrath
				end 
			else
				return nflag, 0.000, 0, 0.000				--dA: buff does not exist on the unit, nflag is default (false)
			end
		else
			print("CLCProt rotation [", rota, "] GetBuff(id) ERROR: id for GetBuff() is nonesuch: ", id, " (sp_name: ", sp_name, ")")
			return nflag, 0.000, 0, 0.000
		end
	else
		print("CLCProt rotation [", rota, "] GetBuff(id) ERROR: id for GetBuff() is invalid: ", id)
		return nflag, 0.000, 0, 0.000
	end
end

local function GetGlobalCooldown(id)						--dA: new function 

	--dA: This function returns (1) the duration, and (2) the time that it will expire
	----: if a GCD is not in progress, the duration is zero, and the time of expiration is zero
	----: if the duration is more than zero but less than the Paladin minimum, that minimum is retured
	----:
	--dA: GCD = 1.5 / ( (1 + (haste% / 100)) * other haste buffs ...)  	[to quote Fouton]
	----:
	---------------------------------------------------------------------------=

	count_getGCD = count_getGCD + 1							--dA: debugging
	local start, duration = 0.000, 0.000

	if id and type(id) == "number" and id > 0 then			--dA: test whether id is nil and positive number 
		
		start, duration = GetSpellCooldown(id)				--dA: returns start = 0 and duration = 0 if GCD is not in progress
--		local xp_gcd = start + duration
--		print("GCD(",id,") st: ",start," dur: ",duration," ct: ",c_time," exp: ",xp_gcd)

		if start and duration then			 				--dA: GetSpellCooldown(id) returns nils if nonesuch spell id
			if duration > 0 and duration < gcdMin then duration = gcdMin end	--dA: this should never be true 
			return duration, start + duration
		else
			print("CLCProt rotation [", rota, "] ERROR: GetGlobalCooldown(id) nonesuch id on call: ", count_getGCD, "id:", id, "start: ", start, " duration: ", duration)
			return 0,0
		end
	end
	print("CLCProt rotation [", rota, "] ERROR: GetGlobalCooldown(id) invalid id on call: ", count_getGCD, "id:", id, " start: ", start, " duration: ", duration)
	return 0,0
end

local function GetUpdates()									--dA: new function to update flags, buffs & debuffs for s2

	-- set the current db
	db = clcprot.db.profile.rotation

	-- time
	local t_now = GetTime()
	later = t_now + howlate 								-- "expiration time" to return if GetCooldown() is not called
	muchlater = later + howlate								-- increase for inactive actions and for error conditions

	-- global cooldown
	if t_now >= gcdExp then									-- gcdExp may be zero (returned by previous call)
		gcdDur, gcdExp = GetGlobalCooldown(gcdId)			-- returns duration & expiration time of current GCD in progress
	end
	
    -- expiration times were previously obtained from GetBuff() and from GetDebuff() via local function GetStatus()
	 r_bas_s = max(0, x_bas_s - t_now)	-- time remaining for the BoG stack (added)
	    r_dp = max(0, x_dp - t_now)		-- time remaining for Divine Purpose proc (added)
	    r_ef = max(0, x_ef - t_now) 	-- time remaining for Eternal Flame buff (added)
	   r_sas = max(0, x_sas - t_now)	-- time remaining for Sacred Shield buff 
	    r_wb = max(0, x_wb - t_now)		-- time remaining for Weakened Blows debuff

	if r_bas_s == 0 then c_bas_s = 0;  end	

	-- check for minimum time to act 
 	if r_dp < db.r_dp_min   then s_dp = false;  end	
	if r_ef < db.r_ef_min   then s_ef = false;  end	
	if r_sas < db.r_sas_min then s_sas = false; end
	if r_wb < db.r_wb_min	then s_wb = false;  end

	-- check flags
	if s_ha and (x_ha - t_now) <= 0 then s_ha = false; end
	if s_aw and (x_aw - t_now) <= 0 then s_aw = false; s_sw = false; end	-- s_sw is assumed false here if s_aw is false
	if s_gc and (x_gc - t_now) <= 0 then s_gc = false; end

end

-- READS ALL THE -INTERESTING- DATA !!!

local function GetStatus()									--dA: revised original source	

	-- set the current db
	db = clcprot.db.profile.rotation

	c_hp = UnitPower("player", SPELL_POWER_HOLY_POWER)		--dA: prior source
	c_mHaste = GetMeleeHaste()								
--	print("CLCProt: rotation [",rota,"] mHaste: ",c_mHaste," sHaste: ",c_sHaste)

	-- global cooldown
	gcdDur, gcdExp = GetGlobalCooldown(gcdId)				--dA: returns duration & expiration time of current GCD in progress
	gcdSoB = max(gcdMin, gcdMax /( 1 + (c_mHaste / 100)))	--dA: evaluate GCD as affected by Sanctity of Battle for UpdateStatus()
	
	c_time = GetTime()
	later = c_time + howlate 								--dA: "expiration time" to return if GetCooldown() is not called
	muchlater = later + howlate								--dA: increase for inactive actions and for error conditions
--[[
	if rota == 1 or rota % 50 == 0 then 
		print("[",rota,"]  GCD: gcdDur: ",gcdDur," gcdExp: ",gcdExp," c_mHaste: ",c_mHaste," c_time: ",c_time) 
--		print("[",rota,"] INFO: c_time: ",c_time," t_rspan: ",t_rspan," later: ",later," gcdDur: ",gcdDur," gcdExp: ",gcdExp," c_hp : ",c_hp," c_mHaste: ",c_mHaste) 
	end 
--]]
	-- the buffs and debuff 								
	--dA: GetBuff() returns (1) true if the buff exists (s_??); (2) the time remaining for the buff (r_??); (3) the number of 
	----: times the buff has been applied; and (4) the time that the buff will expire (x_??) in seconds to millisecond precision,  
	----: a floating-point number, with the same frame of reference as GetTime().
	----: 
	--dA: GetDebuff() returns (1) true if the debuff exists (s_??); (2) the time remaining for the debuff (r_??), (3) the number of 
	----: times the debuff has been applied, and (4) the time that the debuff will expire (x_??) in seconds to millisecond precision,  
	----: a floating-point number, with the same frame of reference as GetTime().
		
				s_aw, _, _, x_aw = GetBuff(awId)		-- avenging wrath		--dA: added	
	_, r_bas_s, c_bas_s, x_bas_s = GetBuff(basId)		-- BoG stack 
			 s_dp, r_dp, _, x_dp = GetBuff(dpId)		-- divine purpose
			 s_ef, r_ef, _, x_ef = GetBuff(efId)		-- eternal flame		--dA: added
				s_gc, _, _, x_gc = GetBuff(gcId)		-- grand crusader	
				s_ha, _, _, x_ha = GetBuff(haId)		-- holy avenger
		  s_sas, r_sas, _, x_sas = GetBuff(sasId)		-- sacred shield
				s_sw, _, _, _, _ = GetBuff(swId)		-- sanctified wrath		--dA: added; s_sw is returned true while SW is in effect
			 s_wb, r_wb, _, x_wb = GetDebuff(buffWB)	-- weakened blows	

	--dA: check for minimum time required to act 
	--dA: minimum time to act for r_bas_s is checked in action efc for BoG expiration with r_bas_s >= db.r_bas_min 

	if r_dp < db.r_dp_min  	then s_dp = false;  end	
	if r_ef < db.r_ef_min   then s_ef = false;  end	
	if r_sas < db.r_sas_min then s_sas = false; end
	if r_wb < db.r_wb_min	then s_wb = false;  end
end

local function GetNextAction()								--dA: the logic is the same, only the names have been changed :-)

	--dA: returns the spell id and "name" of the action with the highest priority 
	----: that has the lowest value for its cooldown expiration time (which may be zero) 
	
	s_efc, s_efg, s_wog, s_wogdp = false, false, false, false	--dA: added; true when the s_???? action is in the queue in use
	local action = ""											--dA: added 
	local cdx = 000.000											--dA: added
	local stop_parse = false									--dA: added
	
	-- set the current db
	db = clcprot.db.profile.rotation

	if useAltPrio then										--dA: useAltPrio is a global variable in clcprot.lua
		q = qalt
		if ( s_efc_qa ) then s_efc = true end				--dA: added
		if prioToggled == "qa" then							--dA: added, prioToggled is a global variable in clcprot.lua 
			print("CLCProt: Alternate Priority is active: ", db.prioAlt)
			prioToggled = "no"
		end
	else
		q = qn
		if ( s_efc_qn ) then s_efc = true end				--dA: added
		if prioToggled == "qn" then							--dA: added, prioToggled is a global variable in clcprot.lua 
			print("CLCProt: Normal Priority is active: ", db.prio)
			prioToggled = "no"
		end
	end
	local n = #q

	-- parse once to get cooldowns, return first spell for which the cooldown has expired, if there is one

	for i = 1, n do
		action = actions[q[i]]							--dA: the value of "action" is an action code specified in the rotation
		if action then 									--dA: added precaution
			cdx = action.GetCD()						--dA: the GetCD method calls GetCooldown() for expiration time of CD
			if cdx then 								--dA: Murphy's Law, general case
				if cdx == 0 then		
					if action.id and q[i] then 			--dA: added precaution
						if action.id > 0 then 			--dA: action.id = 0 is not an error if -no- T6 L90 talent is selected
							return action.id, q[i]		--dA: stops GetNextAction(), then s1 (or s2) = action.id, action = q[i]
						end 
					else
						print("CLCProt rotation [", rota, "] ERROR 3: GetNextAction() 1st parse: invalid data in actions[q[", i, "]] action.id: ", action.id, " q[i]: ", q[i])
					end	
				end
				action.cdx = cdx 						--dA: this trick should be documented 
			else
				print("CLCProt rotation [", rota, "] ERROR 2: GetNextAction() 1st parse: invalid CD expiration for actions[q[", i, "]] action: ", action, " cdx:", cdx)				
			end
		else
			print("CLCProt rotation [", rota, "] ERROR 1: GetNextAction() 1st parse: actions[q[", i, "]] action is ", action)
		end
	end

	-- parse again, return the spell that expires first		--dA: executed when all spells are on cooldown
	
	local minQ = 1
	local minCdx = actions[q[1]].cdx
	repeat
		if minCdx then 
			for i = 2, n do
				local action = actions[q[i]]
				if action then
					if action.cdx then 
						if minCdx > action.cdx then					
							minCdx = action.cdx
							minQ = i
						end
					else
						print("CLCProt rotation [", rota, "] ERROR: GetNextAction() 2nd parse: invalid action.cdx from actions[q[", i, "]] = ", action.cdx)
					end 
				else
					print("CLCProt rotation [", rota, "] ERROR: GetNextAction() 2nd parse: invalid action from actions[q[", i, "]] = ", action)
				end
			end
			stop_parse = true
		else	
			print("CLCProt rotation [", rota, "] ERROR: GetNextAction() 2nd parse - invalid minCdx from actions[q[1]].cdx = ", minCDx)
			minCdx = later									--dA: default CDx on error
		end
	until stop_parse

	return actions[q[minQ]].id, q[minQ]						--dA: the first will be s1, the second will be local action

end

function clcprot.ProtRotation()								--dA: prior source with comments and additions

	rota = rota + 1											--dA: rotation count (added) 

	-- set the current db
	db = clcprot.db.profile.rotation
	s_s1_efc, s_s1_efg = false, false

	GetStatus()							--dA: get current time, expiration times, flags, HP, and other variables 

--	s_s1 = true							--dA: debugging - whether skill chosen is for s1 (added)
	s1 = 0								--dA: valid to compare s1 to action spell Id, but beware whether spell Id is zero
	local action1 = "aaa"
	s1, action1 = GetNextAction()		--dA: s1 is the action's spell ID number, action is a string such as "cons", "sor5", etc.
	actions[action1].UpdateStatus()		--dA: changes the value of c_time & tracks number of HP charges and of BoG stacked

	GetUpdates()						--dA: updates time, expiration times, flags, time left for buffs & debuffs, etc.
  	
--	s_s1 = false						--dA: bebugging - whether skill chosen is for s1 (added)
	s2 = 0
	local action2 = "zzz"
	s2, action2 = GetNextAction()

	--dA: set flag(s) when an AoE spell targeted with a mouse is in s1 and/or s2
	if s1 == lhId or (( s_goc ) and s1 == consId) then s1_twm = true end
	if s2 == lhId or (( s_goc ) and s2 == consId) then s2_twm = true end

	return s1, s2											--dA: return them to clcprot:CheckQueueProt()

end

--- END OF FILE ---------------------------------------------------------------=